# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki

XDG_CACHE_HOME ?= $(HOME)/.cache

GARDEN_CACHE_DIR ?= $(if $(value SNAP_USER_COMMON),$(SNAP_USER_COMMON)/cache,$(XDG_CACHE_HOME)/image-garden)
GARDEN_DL_DIR ?= $(GARDEN_CACHE_DIR)/dl
GARDEN_SHARE_DIR = $(or $(wildcard $(abspath $(dir $(filter %/image-garden.mk,$(MAKEFILE_LIST)))/../share/image-garden/)),.)/
GARDEN_ARCH ?= $(shell uname -m)
GARDEN_ARCH := $(GARDEN_ARCH)

MKPASSWD ?= $(or $(shell command -v mkpasswd),$(error program mkpasswd is required))
GENISOIMAGE ?= $(or $(shell command -v genisoimage),$(error program genisoimage is required))
WGET ?= $(or $(shell command -v wget),$(error wget is required))$(if $(value CI), --quiet)

comma=,
nothing=
space:=$(nothing) $(nothing)

map_to_debian_arch=$(or $(value debian_arch_$1),$1)
debian_arch_x86_64=amd64
debian_arch_aarch64=arm64

.DEFAULT_GOAL := help

.PHONY: all
.PHONY: fetch

.PHONY: clean
clean::
	rm -f *.log

.PHONY:
distclean:: clean

$(GARDEN_DL_DIR):
	mkdir -p $@

%: %.xz
	unxz --keep $<

GARDEN_SYSTEMS ?=

.PHONY: help
help: RANDOM_SYSTEM=$(word $(shell bash -c 'expr $$RANDOM % $(words $(GARDEN_SYSTEMS))'),$(GARDEN_SYSTEMS))
help:
	@echo "Usage: image-garden make <SYSTEM>.{run,qcow2}"
	@echo ""
	@echo "Each SYSTEM has a disk image (.qcow2) and a boot script (.run)."
	@echo "Run 'image-garden make list-systems' for a complete list of systems."
	@echo ""
	@echo "To create a disk image and boot script for an example system use:"
	@echo ""
	@echo "    image-garden make $(RANDOM_SYSTEM).run"
	@echo ""
	@echo "Note that this image was selected at random."

.PHONY: list-systems
list-systems:
	@$(foreach i,$(GARDEN_SYSTEMS),echo $i;)
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
QEMU_IMG ?= $(or $(shell command -v qemu-img),$(error program qemu-img is required))
# NOTE: This is using qemu-system-x86-64 (with the final dash instead of underscore), on purpose.
QEMU_SYSTEM_X86_64 ?= $(if $(value SNAP_INSTANCE_NAME),$$(if [ -z "$${SNAP_INSTANCE_NAME-}" ]; then echo $(SNAP_INSTANCE_NAME).qemu-system-x86-64; else echo qemu-system-x86_64; fi),$(or $(shell command -v qemu-system-x86_64),$(error program qemu-system-x86_64 is required)))
QEMU_SYSTEM_AARCH64 ?= $(if $(value SNAP_INSTANCE_NAME),$$(test -z "$${SNAP_INSTANCE_NAME-}" && echo $(SNAP_INSTANCE_NAME).)qemu-system-aarch64,$(or $(shell command -v qemu-system-aarch64),$(error program qemu-system-aarch64 is required)))
QEMU_SYSTEM_VERSION = $(subst ., ,$(word 4,$(shell $(or $(shell command -v $(QEMU_SYSTEM_X86_64),$(shell command -v QEMU_SYSTEM_AARCH64)) --version)))
# Version of qemu as a triplet major, minor, micro.
QEMU_SYSTEM_VERSION = $(subst ., ,$(word 4,$(shell $(QEMU_SYSTEM_X86_64) --version)))
QEMU_SYSTEM_MAJOR_VERSION = $(firstword $(QEMU_SYSTEM_VERSION))

QEMU_SYSTEM_X86_64_MACHINE = q35
QEMU_SYSTEM_AARCH64_MACHINE = virt

QEMU_SMP_X86_64 = 1
QEMU_SMP_AARCH64 = 1
QEMU_SMP_$(shell uname -m | tr a-z- A-Z_) = 4

QEMU_IMG_SIZE = 64G
QEMU_MEM_SIZE = 2048
QEMU_ENV_QUIRKS ?=

# Full command line for running QEMU with a given EFI firmware.
QEMU_SYSTEM_X86_64_EFI_CMDLINE = \
	$(QEMU_ENV_QUIRKS) \
	$(QEMU_SYSTEM_X86_64) \
	-cpu max \
	-machine $(QEMU_SYSTEM_X86_64_MACHINE)$${QEMU_ACCEL_MACHINE_PROP=,accel=`$(QEMU_SYSTEM_X86_64) -accel help | tail -n +2 | sort | paste -sd :`} \
	$${QEMU_MEM_OPTION=-m $(QEMU_MEM_SIZE)} \
	$${QEMU_SMP_OPTION=-smp $(QEMU_SMP_X86_64)} \
	$${QEMU_DISPLAY_OPTION=-nographic} \
	$${QEMU_BOOT_FIRMWARE_OPTION=-drive if=pflash,format=raw,unit=0,file=$(2),readonly=on \
		-drive if=pflash,format=raw,unit=1,file=$(3)} \
	$${QEMU_STORAGE_OPTION=-drive file=$(1),if=none,format=qcow2,id=drive0,media=disk,cache=writeback,discard=unmap \
		-device virtio-blk,drive=drive0,bootindex=0} \
	$${QEMU_RNG_OPTION=-device virtio-rng-pci} \
	$${QEMU_NET_OPTION=-netdev user,id=netdev0$${QEMU_NETDEV_USER_EXTRA:-} \
		-device virtio-net-pci,netdev=netdev0,id=net0 \
		-smbios type=41,designation=Virtual-LAN,instance=1,kind=ethernet,pcidev=net0}

# Full command line for running QEMU with legacy BIOS.
QEMU_SYSTEM_X86_64_BIOS_CMDLINE = \
	$(QEMU_ENV_QUIRKS) \
	$(QEMU_SYSTEM_X86_64) \
	-cpu max \
	-machine $(QEMU_SYSTEM_X86_64_MACHINE)$${QEMU_ACCEL_MACHINE_PROP=,accel=`$(QEMU_SYSTEM_X86_64) -accel help | tail -n +2 | sort | paste -sd :`} \
	$${QEMU_MEM_OPTION=-m $(QEMU_MEM_SIZE)} \
	$${QEMU_SMP_OPTION=-smp $(QEMU_SMP_X86_64)} \
	$${QEMU_DISPLAY_OPTION=-nographic} \
	$${QEMU_STORAGE_OPTION=-drive file=$(1),if=none,format=qcow2,id=drive0,media=disk,cache=writeback,discard=unmap \
		-device virtio-blk,drive=drive0,bootindex=0} \
	$${QEMU_RNG_OPTION=-device virtio-rng-pci} \
	$${QEMU_NET_OPTION=-netdev user,id=netdev0$${QEMU_NETDEV_USER_EXTRA:-} \
		-device virtio-net-pci,netdev=netdev0,id=net0 \
		-smbios type=41,designation=Virtual-LAN,instance=1,kind=ethernet,pcidev=net0}

QEMU_SYSTEM_AARCH64_EFI_CMDLINE = \
	$(QEMU_ENV_QUIRKS) \
	$(QEMU_SYSTEM_AARCH64) \
	-cpu max \
	-machine $(QEMU_SYSTEM_AARCH64_MACHINE)$${QEMU_ACCEL_MACHINE_PROP=,accel=`$(QEMU_SYSTEM_AARCH64) -accel help | tail -n +2 | sort | paste -sd :`} \
	$${QEMU_MEM_OPTION=-m $(QEMU_MEM_SIZE)} \
	$${QEMU_SMP_OPTION=-smp $(QEMU_SMP_AARCH64)} \
	$${QEMU_DISPLAY_OPTION=-nographic} \
	$${QEMU_BOOT_FIRMWARE_OPTION=-drive if=pflash,format=raw,unit=0,file=$(2),readonly=on \
		-drive if=pflash,format=raw,unit=1,file=$(3)} \
	$${QEMU_STORAGE_OPTION=-drive file=$(1),if=none,format=qcow2,id=drive0,media=disk,cache=writeback,discard=unmap \
		-device virtio-blk,drive=drive0,bootindex=0} \
	$${QEMU_RNG_OPTION=-device virtio-rng-pci} \
	$${QEMU_NET_OPTION=-netdev user,id=netdev0$${QEMU_NETDEV_USER_EXTRA:-} \
		-device virtio-net-pci,netdev=netdev0,id=net0}

# default location on most Debian/Ubuntu systems
QEMU_OVMF_CODE_X86 ?= /usr/share/OVMF/OVMF_CODE.fd /usr/share/OVMF/OVMF_CODE_4M.fd
QEMU_OVMF_VARS_X86 ?= /usr/share/OVMF/OVMF_VARS.fd /usr/share/OVMF/OVMF_VARS_4M.fd

efi-code.x86_64.img: $(or $(firstword $(wildcard $(QEMU_OVMF_CODE_X86))),$(error please install ovmf firmware))
	dd if=$< of=$@ conv=notrunc

efi-vars.x86_64.img: $(or $(firstword $(wildcard $(QEMU_OVMF_VARS_X86))),$(error please install ovmf firmware))
	dd if=$< of=$@ conv=notrunc

QEMU_OVMF_CODE_AARCH64 ?= /usr/share/AAVMF/AAVMF_CODE.fd
QEMU_OVMF_VARS_AARCH64 ?= /usr/share/AAVMF/AAVMF_VARS.fd

efi-code.aarch64.img: $(QEMU_OVMF_CODE_AARCH64)
	dd if=$< of=$@ conv=notrunc

efi-vars.aarch64.img: $(QEMU_OVMF_VARS_AARCH64)
	dd if=$< of=$@ conv=notrunc

# UEFI firmware code and state.
%.aarch64.efi-code.img: | efi-code.aarch64.img
	ln -sf $(firstword $|) $@

%.aarch64.efi-vars.img: efi-vars.aarch64.img
	cp $< $@

%.x86_64.efi-code.img: | efi-code.x86_64.img
	ln -sf $(firstword $|) $@

%.x86_64.efi-vars.img: efi-vars.x86_64.img
	cp $< $@

define BASE_CLOUD_INIT_META_DATA_TEMPLATE
#cloud-config
instance-iid: iid-$1
local-hostname: $(subst .,-,$1)
endef

CLOUD_INIT_META_DATA_TEMPLATE=$(BASE_CLOUD_INIT_META_DATA_TEMPLATE)

define BASE_CLOUD_INIT_USER_DATA_TEMPLATE
#cloud-config
timezone: Europe/Warsaw
power_state:
  delay: now
  mode: poweroff
  message: Powering off
  timeout: 0
  condition: true
hostname: $(subst .,-,$1)
ssh_pwauth: true
users:
- name: $2
  groups: users
  gecos: $2 User
  # Both the username and the password is "$2"
  hashed_passwd: '$(shell echo -n $2 | $(MKPASSWD) --method=SHA-512 --rounds=4096 -s)'
  sudo: ["ALL=(ALL) NOPASSWD:ALL"]
  lock_passwd: false
  shell: /bin/bash
write_files:
- path: /etc/ssh/sshd_config
  permissions: '0600'
  append: true
  content: |
    # image-garden: allow interactive login with password authentication.
    UsePam yes
- path: /etc/cloud/cloud.cfg.d/99-image-garden.cfg
  permissions: '0600'
  content: |
    # image-garden: allow interactive login with password authentication.
    ssh_pwauth: true
runcmd:
- touch /etc/cloud/cloud-init.disabled
endef

CLOUD_INIT_USER_DATA_TEMPLATE=$(BASE_CLOUD_INIT_USER_DATA_TEMPLATE)

# Seed ISOs with cloud-init files.
# The switch -graft-points allows us to have different (fixed) name in the ISO
# and different (variable) names in the local file system. See the manual page
# of genisoimage for details.
%.seed.iso: %.user-data %.meta-data
	$(GENISOIMAGE) \
		-input-charset utf-8 \
		-output $@ \
		-volid CIDATA \
		-joliet \
		-rock \
		-graft-points \
		user-data=$(word 1,$^) \
		meta-data=$(word 2,$^)

# Pick one of the three templates, in order of priority:
# - $(DISTRO)_$(VERSION)_CLOUD_INIT_USER_DATA_TEMPLATE
# - $(DISTRO)_CLOUD_INIT_USER_DATA_TEMPLATE
# - CLOUD_INIT_USER_DATA_TEMPLATE
# This evaluates the name of the template. Use $(call $(call ...)...) to
# evaluate the resulting template.
PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR = $(or $(if $(value $1_$2_CLOUD_INIT_USER_DATA_TEMPLATE),$1_$2_CLOUD_INIT_USER_DATA_TEMPLATE),$(if $(value $1_CLOUD_INIT_USER_DATA_TEMPLATE),$1_CLOUD_INIT_USER_DATA_TEMPLATE),CLOUD_INIT_USER_DATA_TEMPLATE)

.PHONY: clean
clean::
	rm -f $(foreach a,x86_64 aarch64,efi-code.$a.img efi-vars.$a.img)
	rm -f $(foreach a,x86_64 aarch64,*.$a.qcow2 *.$a.efi-code.img *.$a.efi-vars.img)
	rm -f *.seed.iso *.meta-data *.user-data

%.img: %.img.xz
	unxz --keep $<

%.qcow2: %.img
	qemu-img convert -f raw -O qcow2 $< $@
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ALMALINUX_RELEASES += 8

.PHONY: all
all:: almalinux-cloud-8.$(GARDEN_ARCH).qcow2 almalinux-cloud-8.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/almalinux/AlmaLinux-8-GenericCloud-latest.$(GARDEN_ARCH).qcow2

$(GARDEN_DL_DIR)/almalinux/AlmaLinux-8-GenericCloud-latest.x86_64.qcow2 $(GARDEN_DL_DIR)/almalinux/AlmaLinux-8-GenericCloud-latest.aarch64.qcow2: $(GARDEN_DL_DIR)/almalinux/AlmaLinux-8-GenericCloud-latest.%.qcow2: | $(GARDEN_DL_DIR)/almalinux
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://repo.almalinux.org/almalinux/8/cloud/$*/images/AlmaLinux-8-GenericCloud-latest.$*.qcow2)

$(GARDEN_DL_DIR)/almalinux/almalinux-cloud-8.x86_64.qcow2: | $(GARDEN_DL_DIR)/almalinux $(GARDEN_DL_DIR)/almalinux/AlmaLinux-8-GenericCloud-latest.x86_64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/almalinux/almalinux-cloud-8.aarch64.qcow2: | $(GARDEN_DL_DIR)/almalinux $(GARDEN_DL_DIR)/almalinux/AlmaLinux-8-GenericCloud-latest.aarch64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/almalinux/almalinux-cloud-8.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/almalinux/AlmaLinux-8-GenericCloud-latest.*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ALMALINUX_RELEASES += 9

.PHONY: all
all:: almalinux-cloud-9.$(GARDEN_ARCH).qcow2 almalinux-cloud-9.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/almalinux/AlmaLinux-9-GenericCloud-latest.$(GARDEN_ARCH).qcow2

$(GARDEN_DL_DIR)/almalinux/AlmaLinux-9-GenericCloud-latest.x86_64.qcow2 $(GARDEN_DL_DIR)/almalinux/AlmaLinux-9-GenericCloud-latest.aarch64.qcow2: $(GARDEN_DL_DIR)/almalinux/AlmaLinux-9-GenericCloud-latest.%.qcow2: | $(GARDEN_DL_DIR)/almalinux
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://repo.almalinux.org/almalinux/9/cloud/$*/images/AlmaLinux-9-GenericCloud-latest.$*.qcow2)

$(GARDEN_DL_DIR)/almalinux/almalinux-cloud-9.x86_64.qcow2: | $(GARDEN_DL_DIR)/almalinux $(GARDEN_DL_DIR)/almalinux/AlmaLinux-9-GenericCloud-latest.x86_64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/almalinux/almalinux-cloud-9.aarch64.qcow2: | $(GARDEN_DL_DIR)/almalinux $(GARDEN_DL_DIR)/almalinux/AlmaLinux-9-GenericCloud-latest.aarch64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/almalinux/almalinux-cloud-9.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/almalinux/AlmaLinux-9-GenericCloud-latest.*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ALMALINUX_RELEASES ?=
GARDEN_SYSTEMS += $(foreach r,$(ALMALINUX_RELEASES),$(foreach a,x86_64 aarch64,almalinux-cloud-$r.$a))

$(foreach R,$(ALMALINUX_RELEASES),almalinux-cloud-$R.x86_64.run): almalinux-cloud-%.x86_64.run: $(MAKEFILE_LIST) | almalinux-cloud-%.x86_64.qcow2 almalinux-cloud-%.x86_64.efi-code.img almalinux-cloud-%.x86_64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/almalinux" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(ALMALINUX_RELEASES),almalinux-cloud-$R.aarch64.run): almalinux-cloud-%.aarch64.run: $(MAKEFILE_LIST) | almalinux-cloud-%.aarch64.qcow2 almalinux-cloud-%.aarch64.efi-code.img almalinux-cloud-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/almalinux" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(ALMALINUX_RELEASES),almalinux-cloud-$R.x86_64.qcow2): almalinux-cloud-%.x86_64.qcow2: $(GARDEN_DL_DIR)/almalinux/almalinux-cloud-%.x86_64.qcow2 almalinux-cloud-%.seed.iso almalinux-cloud-%.x86_64.efi-code.img almalinux-cloud-%.x86_64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(foreach R,$(ALMALINUX_RELEASES),almalinux-cloud-$R.aarch64.qcow2): almalinux-cloud-%.aarch64.qcow2: $(GARDEN_DL_DIR)/almalinux/almalinux-cloud-%.aarch64.qcow2 almalinux-cloud-%.seed.iso almalinux-cloud-%.aarch64.efi-code.img almalinux-cloud-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(GARDEN_DL_DIR)/almalinux: | $(GARDEN_DL_DIR)
	mkdir -p $@

almalinux-cloud-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,almalinux-$*)
almalinux-cloud-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

almalinux-cloud-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,ALMALINUX,$*),almalinux-$*,almalinux)
almalinux-cloud-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(ALMALINUX_RELEASES),$(foreach A,x86_64 aarch64,almalinux-cloud-$R.$A.qcow2 almalinux-cloud-$R.$A.run))
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ALPINE_RELEASES += 3
ALPINE_3_SUBVER = 21.2

.PHONY: all
all:: alpine-cloud-3.$(GARDEN_ARCH).qcow2 alpine-cloud-3.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/alpine/generic_alpine-3.$(ALPINE_3_SUBVER)-$(GARDEN_ARCH)-uefi-cloudinit-r0.qcow2

$(GARDEN_DL_DIR)/alpine/generic_alpine-3.$(ALPINE_3_SUBVER)-x86_64-uefi-cloudinit-r0.qcow2 $(GARDEN_DL_DIR)/alpine/generic_alpine-3.$(ALPINE_3_SUBVER)-aarch64-uefi-cloudinit-r0.qcow2: $(GARDEN_DL_DIR)/alpine/generic_alpine-3.$(ALPINE_3_SUBVER)-%-uefi-cloudinit-r0.qcow2: | $(GARDEN_DL_DIR)/alpine
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://dl-cdn.alpinelinux.org/alpine/v3.$(firstword $(subst ., ,$(ALPINE_3_SUBVER)))/releases/cloud/generic_alpine-3.$(ALPINE_3_SUBVER)-$*-uefi-cloudinit-r0.qcow2)

$(GARDEN_DL_DIR)/alpine/alpine-cloud-3.x86_64.qcow2: | $(GARDEN_DL_DIR)/alpine $(GARDEN_DL_DIR)/alpine/generic_alpine-3.$(ALPINE_3_SUBVER)-x86_64-uefi-cloudinit-r0.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/alpine/alpine-cloud-3.aarch64.qcow2: | $(GARDEN_DL_DIR)/alpine $(GARDEN_DL_DIR)/alpine/generic_alpine-3.$(ALPINE_3_SUBVER)-aarch64-uefi-cloudinit-r0.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/alpine/alpine-cloud-3.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/alpine/generic_alpine-3.$(ALPINE_3_SUBVER)-*-uefi-cloudinit-r0.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ALPINE_RELEASES ?=
GARDEN_SYSTEMS += $(foreach r,$(ALPINE_RELEASES),$(foreach a,x86_64 aarch64,alpine-cloud-$r.$a))

$(foreach R,$(ALPINE_RELEASES),alpine-cloud-$R.x86_64.run): alpine-cloud-%.x86_64.run: $(MAKEFILE_LIST) | alpine-cloud-%.x86_64.qcow2 alpine-cloud-%.x86_64.efi-code.img alpine-cloud-%.x86_64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/alpine" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(ALPINE_RELEASES),alpine-cloud-$R.aarch64.run): alpine-cloud-%.aarch64.run: $(MAKEFILE_LIST) | alpine-cloud-%.aarch64.qcow2 alpine-cloud-%.aarch64.efi-code.img alpine-cloud-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/alpine" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(ALPINE_RELEASES),alpine-cloud-$R.x86_64.qcow2): alpine-cloud-%.x86_64.qcow2: $(GARDEN_DL_DIR)/alpine/alpine-cloud-%.x86_64.qcow2 alpine-cloud-%.seed.iso alpine-cloud-%.x86_64.efi-code.img alpine-cloud-%.x86_64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(foreach R,$(ALPINE_RELEASES),alpine-cloud-$R.aarch64.qcow2): alpine-cloud-%.aarch64.qcow2: $(GARDEN_DL_DIR)/alpine/alpine-cloud-%.aarch64.qcow2 alpine-cloud-%.seed.iso alpine-cloud-%.aarch64.efi-code.img alpine-cloud-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(GARDEN_DL_DIR)/alpine: | $(GARDEN_DL_DIR)
	mkdir -p $@

alpine-cloud-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,alpine-$*)
alpine-cloud-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

alpine-cloud-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,ALPINE,$*),alpine-$*,alpine)
alpine-cloud-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(ALPINE_RELEASES),$(foreach A,x86_64 aarch64,alpine-cloud-$R.$A.qcow2 alpine-cloud-$R.$A.run))
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Maciek Borzecki
AMAZONLINUX_RELEASES += 2023

.PHONY: all
all:: amazonlinux-cloud-2023.$(GARDEN_ARCH).qcow2 amazonlinux-cloud-2023.$(GARDEN_ARCH).run

# The image URL isn't stable and changes with every release version, e.g.
# https://cdn.amazonlinux.com/al2023/os-images/latest/kvm/al2023-kvm-<unstable-part>-x86_64.xfs.gpt.qcow2
# but the actual image name is stored in SHA256SUMS, so we can extract that

# x86_64 support

AMAZONLINUX_2023_X86_64_FILENAME ?=

$(GARDEN_DL_DIR)/amazonlinux/SHA256SUMS.2023.x86_64: | $(GARDEN_DL_DIR)/amazonlinux
	$(strip $(WGET) \
		--no-clobber \
		--output-document=$@ \
		https://cdn.amazonlinux.com/al2023/os-images/latest/kvm/SHA256SUMS)

$(GARDEN_DL_DIR)/amazonlinux/info.2023.x86_64.mk: $(GARDEN_DL_DIR)/amazonlinux/SHA256SUMS.2023.x86_64
	echo 'AMAZONLINUX_2023_X86_64_FILENAME := $(or $(lastword $(file < $<)),UNKNOWN)' >$@
	touch --reference=$< $@

include $(GARDEN_DL_DIR)/amazonlinux/info.2023.x86_64.mk

ifneq ($(AMAZONLINUX_2023_X86_64_FILENAME),)
AMAZONLINUX_2023_X86_64_URL = https://cdn.amazonlinux.com/al2023/os-images/latest/kvm/$(AMAZONLINUX_2023_X86_64_FILENAME)

$(GARDEN_DL_DIR)/amazonlinux/$(AMAZONLINUX_2023_X86_64_FILENAME): $(GARDEN_DL_DIR)/amazonlinux/SHA256SUMS.2023.x86_64 | $(GARDEN_DL_DIR)/amazonlinux
	$(strip $(WGET) \
			--no-clobber \
			--directory-prefix=$(firstword $|)/ \
			$(AMAZONLINUX_2023_X86_64_URL))

$(GARDEN_DL_DIR)/amazonlinux/amazonlinux-cloud-2023.x86_64.qcow2: | $(GARDEN_DL_DIR)/amazonlinux $(GARDEN_DL_DIR)/amazonlinux/$(AMAZONLINUX_2023_X86_64_FILENAME)
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/amazonlinux/$(AMAZONLINUX_2023_X86_64_FILENAME)

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/amazonlinux/$(AMAZONLINUX_2023_X86_64_FILENAME)

endif

# aarch64 support

AMAZONLINUX_2023_AARCH64_FILENAME ?=

$(GARDEN_DL_DIR)/amazonlinux/SHA256SUMS.2023.aarch64: | $(GARDEN_DL_DIR)/amazonlinux
	$(strip $(WGET) \
		--no-clobber \
		--output-document=$@ \
		https://cdn.amazonlinux.com/al2023/os-images/latest/kvm-arm64/SHA256SUMS)

$(GARDEN_DL_DIR)/amazonlinux/info.2023.aarch64.mk: $(GARDEN_DL_DIR)/amazonlinux/SHA256SUMS.2023.aarch64
	echo 'AMAZONLINUX_2023_AARCH64_FILENAME := $(or $(lastword $(file < $<)),UNKNOWN)' >$@
	touch --reference=$< $@

include $(GARDEN_DL_DIR)/amazonlinux/info.2023.aarch64.mk

ifneq ($(AMAZONLINUX_2023_AARCH64_FILENAME),)
AMAZONLINUX_2023_AARCH64_URL = https://cdn.amazonlinux.com/al2023/os-images/latest/kvm-arm64/$(AMAZONLINUX_2023_AARCH64_FILENAME)

$(GARDEN_DL_DIR)/amazonlinux/$(AMAZONLINUX_2023_AARCH64_FILENAME): $(GARDEN_DL_DIR)/amazonlinux/SHA256SUMS.2023.aarch64 | $(GARDEN_DL_DIR)/amazonlinux
	$(strip $(WGET) \
			--no-clobber \
			--directory-prefix=$(firstword $|)/ \
			$(AMAZONLINUX_2023_AARCH64_URL))

$(GARDEN_DL_DIR)/amazonlinux/amazonlinux-cloud-2023.aarch64.qcow2: | $(GARDEN_DL_DIR)/amazonlinux $(GARDEN_DL_DIR)/amazonlinux/$(AMAZONLINUX_2023_AARCH64_FILENAME)
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: distclean
distclean::
		rm -f $(GARDEN_DL_DIR)/amazonlinux/$(AMAZONLINUX_2023_AARCH64_FILENAME)

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/amazonlinux/$(AMAZONLINUX_2023_AARCH64_FILENAME)

endif

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/amazonlinux/amazonlinux-cloud-2023.*.qcow2
	rm -f $(GARDEN_DL_DIR)/amazonlinux/info.2023.*.mk

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/amazonlinux/SHA256SUMS.2023.*
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Maciek Borzecki
AMAZONLINUX_RELEASES ?=
GARDEN_SYSTEMS += $(foreach r,$(AMAZONLINUX_RELEASES),$(foreach a,x86_64 aarch64,amazonlinux-cloud-$r.$a))

$(foreach R,$(AMAZONLINUX_RELEASES),amazonlinux-cloud-$R.x86_64.run): amazonlinux-cloud-%.x86_64.run: $(MAKEFILE_LIST) | amazonlinux-cloud-%.x86_64.qcow2 amazonlinux-cloud-%.x86_64.efi-code.img amazonlinux-cloud-%.x86_64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/amazonlinux" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(AMAZONLINUX_RELEASES),amazonlinux-cloud-$R.aarch64.run): amazonlinux-cloud-%.aarch64.run: $(MAKEFILE_LIST) | amazonlinux-cloud-%.aarch64.qcow2 amazonlinux-cloud-%.aarch64.efi-code.img amazonlinux-cloud-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/amazonlinux" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(AMAZONLINUX_RELEASES),amazonlinux-cloud-$R.x86_64.qcow2): amazonlinux-cloud-%.x86_64.qcow2: $(GARDEN_DL_DIR)/amazonlinux/amazonlinux-cloud-%.x86_64.qcow2 amazonlinux-cloud-%.seed.iso amazonlinux-cloud-%.x86_64.efi-code.img amazonlinux-cloud-%.x86_64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(foreach R,$(AMAZONLINUX_RELEASES),amazonlinux-cloud-$R.aarch64.qcow2): amazonlinux-cloud-%.aarch64.qcow2: $(GARDEN_DL_DIR)/amazonlinux/amazonlinux-cloud-%.aarch64.qcow2 amazonlinux-cloud-%.seed.iso amazonlinux-cloud-%.aarch64.efi-code.img amazonlinux-cloud-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(GARDEN_DL_DIR)/amazonlinux: | $(GARDEN_DL_DIR)
	mkdir -p $@

amazonlinux-cloud-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,amazonlinux-$*)
amazonlinux-cloud-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

amazonlinux-cloud-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,AMAZONLINUX,$*),amazonlinux-$*,amazonlinux)
amazonlinux-cloud-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(AMAZONLINUX_RELEASES),$(foreach A,x86_64 aarch64,amazonlinux-cloud-$R.$A.qcow2 amazonlinux-cloud-$R.$A.run))
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ifeq ($(GARDEN_ARCH),x86_64)
.PHONY: all
all:: archlinux-cloud.$(GARDEN_ARCH).qcow2 archlinux-cloud.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/archlinux/Arch-Linux-$(GARDEN_ARCH)-cloudimg.qcow2
endif

GARDEN_SYSTEMS += archlinux-cloud.x86_64

archlinux-cloud.x86_64.qcow2: $(GARDEN_DL_DIR)/archlinux/archlinux-cloud.x86_64.qcow2 archlinux-cloud.seed.iso archlinux-cloud.x86_64.efi-code.img archlinux-cloud.x86_64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(GARDEN_DL_DIR)/archlinux/Arch-Linux-x86_64-cloudimg.qcow2: | $(GARDEN_DL_DIR)/archlinux
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://geo.mirror.pkgbuild.com/images/latest/Arch-Linux-x86_64-cloudimg.qcow2)

$(GARDEN_DL_DIR)/archlinux/archlinux-cloud.x86_64.qcow2: | $(GARDEN_DL_DIR)/archlinux $(GARDEN_DL_DIR)/archlinux/Arch-Linux-x86_64-cloudimg.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

archlinux-cloud.x86_64.run: $(MAKEFILE_LIST) | archlinux-cloud.x86_64.qcow2 archlinux-cloud.x86_64.efi-code.img archlinux-cloud.x86_64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/archlinux" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(GARDEN_DL_DIR)/archlinux: | $(GARDEN_DL_DIR)
	mkdir -p $@

archlinux-cloud.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,archlinux)
archlinux-cloud.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

archlinux-cloud.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,ARCHLINUX,),archlinux,archlinux)
archlinux-cloud.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f archlinux-cloud.x86_64.qcow2 archlinux-cloud.x86_64.run
	rm -f $(GARDEN_DL_DIR)/archlinux/archlinux-cloud.x86_64.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/archlinux/Arch-Linux-x86_64-cloudimg.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
CENTOS_RELEASES += 10

.PHONY: all
all:: centos-cloud-10.$(GARDEN_ARCH).qcow2 centos-cloud-10.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-10-latest.$(GARDEN_ARCH).qcow2

$(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-10-latest.aarch64.qcow2 $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-10-latest.x86_64.qcow2: $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-10-latest.%.qcow2: | $(GARDEN_DL_DIR)/centos
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud.centos.org/centos/10-stream/$*/images/CentOS-Stream-GenericCloud-10-latest.$*.qcow2)

$(GARDEN_DL_DIR)/centos/centos-cloud-10.x86_64.qcow2: | $(GARDEN_DL_DIR)/centos $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-10-latest.x86_64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/centos/centos-cloud-10.aarch64.qcow2: | $(GARDEN_DL_DIR)/centos $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-10-latest.aarch64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/centos/centos-cloud-10.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-10-latest.*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
CENTOS_RELEASES += 9

.PHONY: all
all:: centos-cloud-9.$(GARDEN_ARCH).qcow2 centos-cloud-9.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-9-latest.$(GARDEN_ARCH).qcow2

# FIXME: x86-64 image is BIOS, not EFI
$(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-9-latest.aarch64.qcow2 $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-9-latest.x86_64.qcow2: $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-9-latest.%.qcow2: | $(GARDEN_DL_DIR)/centos
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud.centos.org/centos/9-stream/$*/images/CentOS-Stream-GenericCloud-9-latest.$*.qcow2)

$(GARDEN_DL_DIR)/centos/centos-cloud-9.x86_64.qcow2: | $(GARDEN_DL_DIR)/centos $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-9-latest.x86_64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/centos/centos-cloud-9.aarch64.qcow2: | $(GARDEN_DL_DIR)/centos $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-9-latest.aarch64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/centos/centos-cloud-9.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/centos/CentOS-Stream-GenericCloud-9-latest.*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
CENTOS_RELEASES ?=
GARDEN_SYSTEMS += $(foreach r,$(CENTOS_RELEASES),$(foreach a,x86_64 aarch64,centos-cloud-$r.$a))

$(foreach R,$(CENTOS_RELEASES),centos-cloud-$R.x86_64.run): centos-cloud-%.x86_64.run: $(MAKEFILE_LIST) | centos-cloud-%.x86_64.qcow2
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/centos" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_BIOS_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(CENTOS_RELEASES),centos-cloud-$R.aarch64.run): centos-cloud-%.aarch64.run: $(MAKEFILE_LIST) | centos-cloud-%.aarch64.qcow2 centos-cloud-%.aarch64.efi-code.img centos-cloud-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/centos" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(CENTOS_RELEASES),centos-cloud-$R.x86_64.qcow2): centos-cloud-%.x86_64.qcow2: $(GARDEN_DL_DIR)/centos/centos-cloud-%.x86_64.qcow2 centos-cloud-%.seed.iso
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_BIOS_CMDLINE,$@) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(foreach R,$(CENTOS_RELEASES),centos-cloud-$R.aarch64.qcow2): centos-cloud-%.aarch64.qcow2: $(GARDEN_DL_DIR)/centos/centos-cloud-%.aarch64.qcow2 centos-cloud-%.seed.iso centos-cloud-%.aarch64.efi-code.img centos-cloud-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(GARDEN_DL_DIR)/centos: | $(GARDEN_DL_DIR)
	mkdir -p $@

centos-cloud-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,centos-$*)
centos-cloud-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

centos-cloud-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,CENTOS,$*),centos-$*,centos)
centos-cloud-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(CENTOS_RELEASES),$(foreach A,x86_64 aarch64,centos-cloud-$R.$A.qcow2 centos-cloud-$R.$A.run))
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
DEBIAN_RELEASES += 10
DEBIAN_10_CODENAME = buster

.PHONY: all
all:: debian-cloud-10.$(GARDEN_ARCH).qcow2 debian-cloud-10.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/debian/debian-10-generic-$(call map_to_debian_arch,$(GARDEN_ARCH))-daily.qcow2

$(GARDEN_DL_DIR)/debian/debian-10-generic-amd64-daily.qcow2 $(GARDEN_DL_DIR)/debian/debian-10-generic-arm64-daily.qcow2: $(GARDEN_DL_DIR)/debian/debian-10-generic-%-daily.qcow2: | $(GARDEN_DL_DIR)/debian
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud.debian.org/images/cloud/$(DEBIAN_10_CODENAME)/daily/latest/debian-10-generic-$*-daily.qcow2)

$(GARDEN_DL_DIR)/debian/debian-cloud-10.x86_64.qcow2: | $(GARDEN_DL_DIR)/debian $(GARDEN_DL_DIR)/debian/debian-10-generic-amd64-daily.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/debian/debian-cloud-10.aarch64.qcow2: | $(GARDEN_DL_DIR)/debian $(GARDEN_DL_DIR)/debian/debian-10-generic-arm64-daily.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/debian/debian-cloud-10.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/debian/debian-10-generic-*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
DEBIAN_RELEASES += 11
DEBIAN_11_CODENAME = bullseye

.PHONY: all
all:: debian-cloud-11.$(GARDEN_ARCH).qcow2 debian-cloud-11.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/debian/debian-11-genericcloud-$(call map_to_debian_arch,$(GARDEN_ARCH))-daily.qcow2

$(GARDEN_DL_DIR)/debian/debian-11-genericcloud-amd64-daily.qcow2 $(GARDEN_DL_DIR)/debian/debian-11-genericcloud-arm64-daily.qcow2: $(GARDEN_DL_DIR)/debian/debian-11-genericcloud-%-daily.qcow2: | $(GARDEN_DL_DIR)/debian
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud.debian.org/images/cloud/$(DEBIAN_11_CODENAME)/daily/latest/debian-11-genericcloud-$*-daily.qcow2)

$(GARDEN_DL_DIR)/debian/debian-cloud-11.x86_64.qcow2: | $(GARDEN_DL_DIR)/debian $(GARDEN_DL_DIR)/debian/debian-11-genericcloud-amd64-daily.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/debian/debian-cloud-11.aarch64.qcow2: | $(GARDEN_DL_DIR)/debian $(GARDEN_DL_DIR)/debian/debian-11-genericcloud-arm64-daily.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/debian/debian-cloud-11.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/debian/debian-11-genericcloud-*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
DEBIAN_RELEASES += 12
DEBIAN_12_CODENAME = bookworm

.PHONY: all
all:: debian-cloud-12.$(GARDEN_ARCH).qcow2 debian-cloud-12.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/debian/debian-12-genericcloud-$(call map_to_debian_arch,$(GARDEN_ARCH))-daily.qcow2

$(GARDEN_DL_DIR)/debian/debian-12-genericcloud-amd64-daily.qcow2 $(GARDEN_DL_DIR)/debian/debian-12-genericcloud-arm64-daily.qcow2: $(GARDEN_DL_DIR)/debian/debian-12-genericcloud-%-daily.qcow2: | $(GARDEN_DL_DIR)/debian
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud.debian.org/images/cloud/$(DEBIAN_12_CODENAME)/daily/latest/debian-12-genericcloud-$*-daily.qcow2)

$(GARDEN_DL_DIR)/debian/debian-cloud-12.x86_64.qcow2: | $(GARDEN_DL_DIR)/debian $(GARDEN_DL_DIR)/debian/debian-12-genericcloud-amd64-daily.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/debian/debian-cloud-12.aarch64.qcow2: | $(GARDEN_DL_DIR)/debian $(GARDEN_DL_DIR)/debian/debian-12-genericcloud-arm64-daily.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/debian/debian-cloud-12.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/debian/debian-12-genericcloud-*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
DEBIAN_RELEASES += 13
DEBIAN_13_CODENAME = trixie

.PHONY: all
all:: debian-cloud-13.$(GARDEN_ARCH).qcow2 debian-cloud-13.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/debian/debian-13-genericcloud-$(call map_to_debian_arch,$(GARDEN_ARCH))-daily.qcow2

$(GARDEN_DL_DIR)/debian/debian-13-genericcloud-amd64-daily.qcow2 $(GARDEN_DL_DIR)/debian/debian-13-genericcloud-arm64-daily.qcow2: $(GARDEN_DL_DIR)/debian/debian-13-genericcloud-%-daily.qcow2: | $(GARDEN_DL_DIR)/debian
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud.debian.org/images/cloud/$(DEBIAN_13_CODENAME)/daily/latest/debian-13-genericcloud-$*-daily.qcow2)

$(GARDEN_DL_DIR)/debian/debian-cloud-13.x86_64.qcow2: | $(GARDEN_DL_DIR)/debian $(GARDEN_DL_DIR)/debian/debian-13-genericcloud-amd64-daily.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/debian/debian-cloud-13.aarch64.qcow2: | $(GARDEN_DL_DIR)/debian $(GARDEN_DL_DIR)/debian/debian-13-genericcloud-arm64-daily.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/debian/debian-cloud-13.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/debian/debian-13-genericcloud-*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
DEBIAN_RELEASES += sid
DEBIAN_sid_CODENAME = sid

.PHONY: all
all:: debian-cloud-sid.$(GARDEN_ARCH).qcow2 debian-cloud-sid.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/debian/debian-sid-genericcloud-$(call map_to_debian_arch,$(GARDEN_ARCH))-daily.qcow2

$(GARDEN_DL_DIR)/debian/debian-sid-genericcloud-amd64-daily.qcow2 $(GARDEN_DL_DIR)/debian/debian-sid-genericcloud-arm64-daily.qcow2: $(GARDEN_DL_DIR)/debian/debian-sid-genericcloud-%-daily.qcow2: | $(GARDEN_DL_DIR)/debian
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud.debian.org/images/cloud/$(DEBIAN_sid_CODENAME)/daily/latest/debian-sid-genericcloud-$*-daily.qcow2)

$(GARDEN_DL_DIR)/debian/debian-cloud-sid.x86_64.qcow2: | $(GARDEN_DL_DIR)/debian $(GARDEN_DL_DIR)/debian/debian-sid-genericcloud-amd64-daily.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/debian/debian-cloud-sid.aarch64.qcow2: | $(GARDEN_DL_DIR)/debian $(GARDEN_DL_DIR)/debian/debian-sid-genericcloud-arm64-daily.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/debian/debian-cloud-sid.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/debian/debian-sid-genericcloud-*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
DEBIAN_RELEASES ?=
GARDEN_SYSTEMS += $(foreach r,$(DEBIAN_RELEASES),$(foreach a,x86_64 aarch64,debian-cloud-$r.$a))

$(foreach R,$(DEBIAN_RELEASES),debian-cloud-$R.x86_64.run): debian-cloud-%.x86_64.run: $(MAKEFILE_LIST) | debian-cloud-%.x86_64.qcow2 debian-cloud-%.x86_64.efi-code.img debian-cloud-%.x86_64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/debian" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(DEBIAN_RELEASES),debian-cloud-$R.aarch64.run): debian-cloud-%.aarch64.run: $(MAKEFILE_LIST) | debian-cloud-%.aarch64.qcow2 debian-cloud-%.aarch64.efi-code.img debian-cloud-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/debian" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

# Debian cloud images do not contain a standalone installer that you only need
# during the setup process. Instead each image can be used as a read-only base
# for a read-write template.
$(foreach R,$(DEBIAN_RELEASES),debian-cloud-$R.x86_64.qcow2): debian-cloud-%.x86_64.qcow2: $(GARDEN_DL_DIR)/debian/debian-cloud-%.x86_64.qcow2 debian-cloud-%.seed.iso debian-cloud-%.x86_64.efi-code.img debian-cloud-%.x86_64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

# Replicated because we have more releases than architectures and architecture-specific quirks are easier to express.
$(foreach R,$(DEBIAN_RELEASES),debian-cloud-$R.aarch64.qcow2): debian-cloud-%.aarch64.qcow2: $(GARDEN_DL_DIR)/debian/debian-cloud-%.aarch64.qcow2 debian-cloud-%.seed.iso debian-cloud-%.aarch64.efi-code.img debian-cloud-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

# Debian specific downloads.
$(GARDEN_DL_DIR)/debian: | $(GARDEN_DL_DIR)
	mkdir -p $@

debian-cloud-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,debian-$*)
debian-cloud-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

debian-cloud-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,DEBIAN,$*),debian-$*,debian)
debian-cloud-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(DEBIAN_RELEASES),$(foreach A,x86_64 aarch64,debian-cloud-$R.$A.qcow2 debian-cloud-$R.$A.run))
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
FEDORA_RELEASES += 38
FEDORA_38_SUBVER = 1.6

.PHONY: all
all:: fedora-cloud-38.$(GARDEN_ARCH).qcow2 fedora-cloud-38.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-38-$(FEDORA_38_SUBVER).$(GARDEN_ARCH).qcow2

$(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-38-$(FEDORA_38_SUBVER).x86_64.qcow2 $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-38-$(FEDORA_38_SUBVER).aarch64.qcow2: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-38-$(FEDORA_38_SUBVER).%.qcow2: | $(GARDEN_DL_DIR)/fedora
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://download.fedoraproject.org/pub/fedora/linux/releases/38/Cloud/$*/images/Fedora-Cloud-Base-38-$(FEDORA_38_SUBVER).$*.qcow2)

$(GARDEN_DL_DIR)/fedora/fedora-cloud-base-38.%.qcow2: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-38-$(FEDORA_38_SUBVER).%.qcow2 | $(GARDEN_DL_DIR)/fedora
	ln -sf $(notdir $<) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/fedora/fedora-cloud-base-38.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-38-$(FEDORA_38_SUBVER).*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
FEDORA_RELEASES += 39
FEDORA_39_SUBVER = 1.5

.PHONY: all
all:: fedora-cloud-39.$(GARDEN_ARCH).qcow2 fedora-cloud-39.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-39-$(FEDORA_39_SUBVER).$(GARDEN_ARCH).qcow2

$(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-39-$(FEDORA_39_SUBVER).x86_64.qcow2 $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-39-$(FEDORA_39_SUBVER).aarch64.qcow2: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-39-$(FEDORA_39_SUBVER).%.qcow2: | $(GARDEN_DL_DIR)/fedora
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://download.fedoraproject.org/pub/fedora/linux/releases/39/Cloud/$*/images/Fedora-Cloud-Base-39-$(FEDORA_39_SUBVER).$*.qcow2)

$(GARDEN_DL_DIR)/fedora/fedora-cloud-base-39.%.qcow2: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-39-$(FEDORA_39_SUBVER).%.qcow2 | $(GARDEN_DL_DIR)/fedora
	ln -sf $(notdir $<) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/fedora/fedora-cloud-base-39.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-39-$(FEDORA_39_SUBVER).*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
FEDORA_RELEASES += 40
FEDORA_40_SUBVER = 1.14

.PHONY: all
all:: fedora-cloud-40.$(GARDEN_ARCH).qcow2 fedora-cloud-40.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic.$(GARDEN_ARCH)-40-$(FEDORA_40_SUBVER).qcow2

$(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic.x86_64-40-$(FEDORA_40_SUBVER).qcow2 $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic.aarch64-40-$(FEDORA_40_SUBVER).qcow2: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic.%-40-$(FEDORA_40_SUBVER).qcow2: | $(GARDEN_DL_DIR)/fedora
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://download.fedoraproject.org/pub/fedora/linux/releases/40/Cloud/$*/images/Fedora-Cloud-Base-Generic.$*-40-$(FEDORA_40_SUBVER).qcow2)

$(GARDEN_DL_DIR)/fedora/fedora-cloud-base-40.%.qcow2: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic.%-40-$(FEDORA_40_SUBVER).qcow2 | $(GARDEN_DL_DIR)/fedora
	ln -sf $(notdir $<) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/fedora/fedora-cloud-base-40.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic.*-40-$(FEDORA_40_SUBVER).qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
FEDORA_RELEASES += 41
FEDORA_41_SUBVER = 1.4

.PHONY: all
all:: fedora-cloud-41.$(GARDEN_ARCH).qcow2 fedora-cloud-41.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic-41-$(FEDORA_41_SUBVER).$(GARDEN_ARCH).qcow2

$(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic-41-$(FEDORA_41_SUBVER).x86_64.qcow2 $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic-41-$(FEDORA_41_SUBVER).aarch64.qcow2: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic-41-$(FEDORA_41_SUBVER).%.qcow2: | $(GARDEN_DL_DIR)/fedora
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://download.fedoraproject.org/pub/fedora/linux/releases/41/Cloud/$*/images/Fedora-Cloud-Base-Generic-41-$(FEDORA_41_SUBVER).$*.qcow2)

$(GARDEN_DL_DIR)/fedora/fedora-cloud-base-41.%.qcow2: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic-41-$(FEDORA_41_SUBVER).%.qcow2 | $(GARDEN_DL_DIR)/fedora
	ln -sf $(notdir $<) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/fedora/fedora-cloud-base-41.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic.*-41-$(FEDORA_41_SUBVER).qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
FEDORA_RELEASES += 42
FEDORA_42_BETA_SUBVER = 1.1

.PHONY: all
all:: fedora-cloud-42.$(GARDEN_ARCH).qcow2 fedora-cloud-42.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic-42-$(FEDORA_42_BETA_SUBVER).$(GARDEN_ARCH).qcow2

$(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic-42-$(FEDORA_42_BETA_SUBVER).x86_64.qcow2 $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic-42-$(FEDORA_42_BETA_SUBVER).aarch64.qcow2: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic-42-$(FEDORA_42_BETA_SUBVER).%.qcow2: | $(GARDEN_DL_DIR)/fedora
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://download.fedoraproject.org/pub/fedora/linux/releases/42/Cloud/$*/images/Fedora-Cloud-Base-Generic-42-$(FEDORA_42_BETA_SUBVER).$*.qcow2)

$(GARDEN_DL_DIR)/fedora/fedora-cloud-base-42.%.qcow2: $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic-42-$(FEDORA_42_BETA_SUBVER).%.qcow2 | $(GARDEN_DL_DIR)/fedora
	ln -sf $(notdir $<) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/fedora/fedora-cloud-base-42.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/fedora/Fedora-Cloud-Base-Generic.*-42-$(FEDORA_42_BETA_SUBVER).qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
FEDORA_RELEASES ?=
GARDEN_SYSTEMS += $(foreach r,$(FEDORA_RELEASES),$(foreach a,x86_64 aarch64,fedora-cloud-$r.$a))

$(foreach R,$(FEDORA_RELEASES),fedora-cloud-$R.x86_64.run): fedora-cloud-%.x86_64.run: $(MAKEFILE_LIST) | fedora-cloud-%.x86_64.qcow2 fedora-cloud-%.x86_64.efi-code.img fedora-cloud-%.x86_64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/fedora" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(FEDORA_RELEASES),fedora-cloud-$R.aarch64.run): fedora-cloud-%.aarch64.run: $(MAKEFILE_LIST) | fedora-cloud-%.aarch64.qcow2 fedora-cloud-%.aarch64.efi-code.img fedora-cloud-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/fedora" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

# Fedora cloud images do not contain a standalone installer that you only need
# during the setup process. Instead each image can be used as a read-only base
# for a read-write template.
$(foreach R,$(FEDORA_RELEASES),fedora-cloud-$R.x86_64.qcow2): fedora-cloud-%.x86_64.qcow2: $(GARDEN_DL_DIR)/fedora/fedora-cloud-base-%.x86_64.qcow2 fedora-cloud-%.seed.iso fedora-cloud-%.x86_64.efi-code.img fedora-cloud-%.x86_64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

# Replicated because we have more releases than architectures and architecture-specific quirks are easier to express.
$(foreach R,$(FEDORA_RELEASES),fedora-cloud-$R.aarch64.qcow2): fedora-cloud-%.aarch64.qcow2: $(GARDEN_DL_DIR)/fedora/fedora-cloud-base-%.aarch64.qcow2 fedora-cloud-%.seed.iso fedora-cloud-%.aarch64.efi-code.img fedora-cloud-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

# Fedora specific downloads.
$(GARDEN_DL_DIR)/fedora: | $(GARDEN_DL_DIR)
	mkdir -p $@

fedora-cloud-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,fedora-$*)
fedora-cloud-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

fedora-cloud-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,FEDORA,$*),fedora-$*,fedora)
fedora-cloud-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(FEDORA_RELEASES),$(foreach A,x86_64 aarch64,fedora-cloud-$R.$A.qcow2 fedora-cloud-$R.$A.run))
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
OPENSUSE_RELEASES += 15.4

.PHONY: all
all:: opensuse-cloud-15.4.$(GARDEN_ARCH).qcow2 opensuse-cloud-15.4.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.4.$(GARDEN_ARCH)-NoCloud.qcow2

$(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.4.x86_64-NoCloud.qcow2 $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.4.aarch64-NoCloud.qcow2: $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.4.%-NoCloud.qcow2: | $(GARDEN_DL_DIR)/opensuse
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://download.opensuse.org/repositories/Cloud:/Images:/Leap_15.4/images/openSUSE-Leap-15.4.$*-NoCloud.qcow2)

$(GARDEN_DL_DIR)/opensuse/opensuse-cloud-15.4.x86_64.qcow2: | $(GARDEN_DL_DIR)/opensuse $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.4.x86_64-NoCloud.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/opensuse/opensuse-cloud-15.4.aarch64.qcow2: | $(GARDEN_DL_DIR)/opensuse $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.4.aarch64-NoCloud.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/opensuse/opensuse-cloud-15.4.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.4.*-NoCloud.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
OPENSUSE_RELEASES += 15.5

.PHONY: all
all:: opensuse-cloud-15.5.$(GARDEN_ARCH).qcow2 opensuse-cloud-15.5.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.5.$(GARDEN_ARCH)-NoCloud.qcow2

$(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.5.x86_64-NoCloud.qcow2 $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.5.aarch64-NoCloud.qcow2: $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.5.%-NoCloud.qcow2: | $(GARDEN_DL_DIR)/opensuse
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://download.opensuse.org/repositories/Cloud:/Images:/Leap_15.5/images/openSUSE-Leap-15.5.$*-NoCloud.qcow2)

$(GARDEN_DL_DIR)/opensuse/opensuse-cloud-15.5.x86_64.qcow2: | $(GARDEN_DL_DIR)/opensuse $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.5.x86_64-NoCloud.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/opensuse/opensuse-cloud-15.5.aarch64.qcow2: | $(GARDEN_DL_DIR)/opensuse $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.5.aarch64-NoCloud.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/opensuse/opensuse-cloud-15.5.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.5.*-NoCloud.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
OPENSUSE_RELEASES += 15.6

.PHONY: all
all:: opensuse-cloud-15.6.$(GARDEN_ARCH).qcow2 opensuse-cloud-15.6.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.6.$(GARDEN_ARCH)-NoCloud.qcow2

$(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.6.x86_64-NoCloud.qcow2 $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.6.aarch64-NoCloud.qcow2: $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.6.%-NoCloud.qcow2: | $(GARDEN_DL_DIR)/opensuse
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://download.opensuse.org/repositories/Cloud:/Images:/Leap_15.6/images/openSUSE-Leap-15.6.$*-NoCloud.qcow2)

$(GARDEN_DL_DIR)/opensuse/opensuse-cloud-15.6.x86_64.qcow2: | $(GARDEN_DL_DIR)/opensuse $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.6.x86_64-NoCloud.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/opensuse/opensuse-cloud-15.6.aarch64.qcow2: | $(GARDEN_DL_DIR)/opensuse $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.6.aarch64-NoCloud.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/opensuse/opensuse-cloud-15.6.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/opensuse/openSUSE-Leap-15.6.*-NoCloud.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
OPENSUSE_RELEASES += tumbleweed

.PHONY: all
all:: opensuse-cloud-tumbleweed.$(GARDEN_ARCH).qcow2 opensuse-cloud-tumbleweed.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/opensuse/openSUSE-Tumbleweed-Minimal-VM.$(GARDEN_ARCH)-Cloud.qcow2

$(GARDEN_DL_DIR)/opensuse/openSUSE-Tumbleweed-Minimal-VM.aarch64-Cloud.qcow2: | $(GARDEN_DL_DIR)/opensuse
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://download.opensuse.org/repositories/Virtualization:/Appliances:/Images:/openSUSE-Tumbleweed/openSUSE_Tumbleweed_ARM/openSUSE-Tumbleweed-Minimal-VM.aarch64-Cloud.qcow2)

$(GARDEN_DL_DIR)/opensuse/openSUSE-Tumbleweed-Minimal-VM.x86_64-Cloud.qcow2: | $(GARDEN_DL_DIR)/opensuse
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://download.opensuse.org/repositories/Virtualization:/Appliances:/Images:/openSUSE-Tumbleweed/openSUSE_Tumbleweed/openSUSE-Tumbleweed-Minimal-VM.x86_64-Cloud.qcow2)

$(GARDEN_DL_DIR)/opensuse/opensuse-cloud-tumbleweed.x86_64.qcow2: | $(GARDEN_DL_DIR)/opensuse $(GARDEN_DL_DIR)/opensuse/openSUSE-Tumbleweed-Minimal-VM.x86_64-Cloud.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/opensuse/opensuse-cloud-tumbleweed.aarch64.qcow2: | $(GARDEN_DL_DIR)/opensuse $(GARDEN_DL_DIR)/opensuse/openSUSE-Tumbleweed-Minimal-VM.aarch64-Cloud.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/opensuse/opensuse-cloud-tumbleweed.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/opensuse/openSUSE-Tumbleweed-Minimal-VM.*-Cloud.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
OPENSUSE_RELEASES ?=
GARDEN_SYSTEMS += $(foreach r,$(OPENSUSE_RELEASES),$(foreach a,x86_64 aarch64,opensuse-cloud-$r.$a))

$(foreach R,$(OPENSUSE_RELEASES),opensuse-cloud-$R.x86_64.run): opensuse-cloud-%.x86_64.run: $(MAKEFILE_LIST) | opensuse-cloud-%.x86_64.qcow2 opensuse-cloud-%.x86_64.efi-code.img opensuse-cloud-%.x86_64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/opensuse" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(OPENSUSE_RELEASES),opensuse-cloud-$R.aarch64.run): opensuse-cloud-%.aarch64.run: $(MAKEFILE_LIST) | opensuse-cloud-%.aarch64.qcow2 opensuse-cloud-%.aarch64.efi-code.img opensuse-cloud-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/opensuse" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

# Debian cloud images do not contain a standalone installer that you only need
# during the setup process. Instead each image can be used as a read-only base
# for a read-write template.
$(foreach R,$(OPENSUSE_RELEASES),opensuse-cloud-$R.x86_64.qcow2): opensuse-cloud-%.x86_64.qcow2: $(GARDEN_DL_DIR)/opensuse/opensuse-cloud-%.x86_64.qcow2 opensuse-cloud-%.seed.iso opensuse-cloud-%.x86_64.efi-code.img opensuse-cloud-%.x86_64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

# Replicated because we have more releases than architectures and architecture-specific quirks are easier to express.
$(foreach R,$(OPENSUSE_RELEASES),opensuse-cloud-$R.aarch64.qcow2): opensuse-cloud-%.aarch64.qcow2: $(GARDEN_DL_DIR)/opensuse/opensuse-cloud-%.aarch64.qcow2 opensuse-cloud-%.seed.iso opensuse-cloud-%.aarch64.efi-code.img opensuse-cloud-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(GARDEN_DL_DIR)/opensuse: | $(GARDEN_DL_DIR)
	mkdir -p $@

opensuse-cloud-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,opensuse-$*)
opensuse-cloud-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

opensuse-cloud-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,OPENSUSE,$*),opensuse-$*,opensuse)
opensuse-cloud-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(OPENSUSE_RELEASES),$(foreach A,x86_64 aarch64,opensuse-cloud-$R.$A.qcow2 opensuse-cloud-$R.$A.run))
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ORACLE_RELEASES += 8

.PHONY: all
all:: oracle-cloud-8.$(GARDEN_ARCH).qcow2 oracle-cloud-8.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/oracle/OL8U10_x86_64-kvm-b237.qcow2
fetch: $(GARDEN_DL_DIR)/oracle/OL8U10_aarch64-kvm-cloud-b100.qcow2

$(GARDEN_DL_DIR)/oracle/OL8U10_x86_64-kvm-b237.qcow2: | $(GARDEN_DL_DIR)/oracle
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://yum.oracle.com/templates/OracleLinux/OL8/u10/x86_64/OL8U10_x86_64-kvm-b237.qcow2)

$(GARDEN_DL_DIR)/oracle/OL8U10_aarch64-kvm-cloud-b100.qcow2: | $(GARDEN_DL_DIR)/oracle
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://yum.oracle.com/templates/OracleLinux/OL8/u10/aarch64/OL8U10_aarch64-kvm-cloud-b100.qcow2)

$(GARDEN_DL_DIR)/oracle/oracle-cloud-8.x86_64.qcow2: $(MAKEFILE_LIST) | $(GARDEN_DL_DIR)/oracle $(GARDEN_DL_DIR)/oracle/OL8U10_x86_64-kvm-b237.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/oracle/oracle-cloud-8.aarch64.qcow2: $(MAKEFILE_LIST) | $(GARDEN_DL_DIR)/oracle $(GARDEN_DL_DIR)/oracle/OL8U10_aarch64-kvm-cloud-b100.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/oracle/oracle-cloud-8.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/oracle/OL8U10_x86_64-kvm-b*.qcow2 $(GARDEN_DL_DIR)/oracle/OL8U10_aarch64-kvm-cloud-b*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ORACLE_RELEASES += 9

.PHONY: all
all:: oracle-cloud-9.$(GARDEN_ARCH).qcow2 oracle-cloud-9.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/oracle/OL9U5_aarch64-kvm-cloud-b117.qcow2
fetch: $(GARDEN_DL_DIR)/oracle/OL9U5_x86_64-kvm-b253.qcow2

$(GARDEN_DL_DIR)/oracle/OL9U5_x86_64-kvm-b253.qcow2: | $(GARDEN_DL_DIR)/oracle
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://yum.oracle.com/templates/OracleLinux/OL9/u5/x86_64/OL9U5_x86_64-kvm-b253.qcow2)

$(GARDEN_DL_DIR)/oracle/OL9U5_aarch64-kvm-cloud-b117.qcow2: | $(GARDEN_DL_DIR)/oracle
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://yum.oracle.com/templates/OracleLinux/OL9/u5/aarch64/OL9U5_aarch64-kvm-cloud-b117.qcow2)

$(GARDEN_DL_DIR)/oracle/oracle-cloud-9.x86_64.qcow2: $(MAKEFILE_LIST) | $(GARDEN_DL_DIR)/oracle $(GARDEN_DL_DIR)/oracle/OL9U5_x86_64-kvm-b253.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/oracle/oracle-cloud-9.aarch64.qcow2: $(MAKEFILE_LIST) | $(GARDEN_DL_DIR)/oracle $(GARDEN_DL_DIR)/oracle/OL9U5_aarch64-kvm-cloud-b117.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/oracle/oracle-cloud-9.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/oracle/OL9U5_x86_64-kvm-b*.qcow2 $(GARDEN_DL_DIR)/oracle/OL9U5_aarch64-kvm-cloud-b*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ORACLE_RELEASES ?=
GARDEN_SYSTEMS += $(foreach r,$(ORACLE_RELEASES),$(foreach a,x86_64 aarch64,oracle-cloud-$r.$a))

$(foreach R,$(ORACLE_RELEASES),oracle-cloud-$R.x86_64.run): oracle-cloud-%.x86_64.run: $(MAKEFILE_LIST) | oracle-cloud-%.x86_64.qcow2
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/oracle" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_BIOS_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(ORACLE_RELEASES),oracle-cloud-$R.aarch64.run): oracle-cloud-%.aarch64.run: $(MAKEFILE_LIST) | oracle-cloud-%.aarch64.qcow2 oracle-cloud-%.aarch64.efi-code.img oracle-cloud-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/oracle" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(ORACLE_RELEASES),oracle-cloud-$R.x86_64.qcow2): oracle-cloud-%.x86_64.qcow2: $(GARDEN_DL_DIR)/oracle/oracle-cloud-%.x86_64.qcow2 oracle-cloud-%.seed.iso
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_BIOS_CMDLINE,$@) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(foreach R,$(ORACLE_RELEASES),oracle-cloud-$R.aarch64.qcow2): oracle-cloud-%.aarch64.qcow2: $(GARDEN_DL_DIR)/oracle/oracle-cloud-%.aarch64.qcow2 oracle-cloud-%.seed.iso oracle-cloud-%.aarch64.efi-code.img oracle-cloud-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(GARDEN_DL_DIR)/oracle: | $(GARDEN_DL_DIR)
	mkdir -p $@

oracle-cloud-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,oracle-$*)
oracle-cloud-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

oracle-cloud-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,ORACLE,$*),oracle-$*,oracle)
oracle-cloud-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(ORACLE_RELEASES),$(foreach A,x86_64 aarch64,oracle-cloud-$R.$A.qcow2 oracle-cloud-$R.$A.run))
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ROCKY_RELEASES += 8

.PHONY: all
all:: rocky-cloud-8.$(GARDEN_ARCH).qcow2 rocky-cloud-8.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/rocky/Rocky-8-GenericCloud-Base.latest.$(GARDEN_ARCH).qcow2

$(GARDEN_DL_DIR)/rocky/Rocky-8-GenericCloud-Base.latest.x86_64.qcow2 $(GARDEN_DL_DIR)/rocky/Rocky-8-GenericCloud-Base.latest.aarch64.qcow2: $(GARDEN_DL_DIR)/rocky/Rocky-8-GenericCloud-Base.latest.%.qcow2  : | $(GARDEN_DL_DIR)/rocky
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://dl.rockylinux.org/pub/rocky/8/images/$*/Rocky-8-GenericCloud-Base.latest.$*.qcow2)

$(GARDEN_DL_DIR)/rocky/rocky-cloud-8.x86_64.qcow2: | $(GARDEN_DL_DIR)/rocky $(GARDEN_DL_DIR)/rocky/Rocky-8-GenericCloud-Base.latest.x86_64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/rocky/rocky-cloud-8.aarch64.qcow2: | $(GARDEN_DL_DIR)/rocky $(GARDEN_DL_DIR)/rocky/Rocky-8-GenericCloud-Base.latest.aarch64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/rocky/rocky-cloud-8.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/rocky/Rocky-8-GenericCloud-Base.latest.*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ROCKY_RELEASES += 9

.PHONY: all
all:: rocky-cloud-9.$(GARDEN_ARCH).qcow2 rocky-cloud-9.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/rocky/Rocky-9-GenericCloud-Base.latest.$(GARDEN_ARCH).qcow2

$(GARDEN_DL_DIR)/rocky/Rocky-9-GenericCloud-Base.latest.x86_64.qcow2 $(GARDEN_DL_DIR)/rocky/Rocky-9-GenericCloud-Base.latest.aarch64.qcow2: $(GARDEN_DL_DIR)/rocky/Rocky-9-GenericCloud-Base.latest.%.qcow2  : | $(GARDEN_DL_DIR)/rocky
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://dl.rockylinux.org/pub/rocky/9/images/$*/Rocky-9-GenericCloud-Base.latest.$*.qcow2)

$(GARDEN_DL_DIR)/rocky/rocky-cloud-9.x86_64.qcow2: | $(GARDEN_DL_DIR)/rocky $(GARDEN_DL_DIR)/rocky/Rocky-9-GenericCloud-Base.latest.x86_64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/rocky/rocky-cloud-9.aarch64.qcow2: | $(GARDEN_DL_DIR)/rocky $(GARDEN_DL_DIR)/rocky/Rocky-9-GenericCloud-Base.latest.aarch64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/rocky/rocky-cloud-9.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/rocky/Rocky-9-GenericCloud-Base.latest.*.qcow2
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
ROCKY_RELEASES ?=
GARDEN_SYSTEMS += $(foreach r,$(ROCKY_RELEASES),$(foreach a,x86_64 aarch64,rocky-cloud-$r.$a))

$(foreach R,$(ROCKY_RELEASES),rocky-cloud-$R.x86_64.run): rocky-cloud-%.x86_64.run: $(MAKEFILE_LIST) | rocky-cloud-%.x86_64.qcow2 rocky-cloud-%.x86_64.efi-code.img rocky-cloud-%.x86_64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/rocky" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(ROCKY_RELEASES),rocky-cloud-$R.aarch64.run): rocky-cloud-%.aarch64.run: $(MAKEFILE_LIST) | rocky-cloud-%.aarch64.qcow2 rocky-cloud-%.aarch64.efi-code.img rocky-cloud-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/rocky" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(ROCKY_RELEASES),rocky-cloud-$R.x86_64.qcow2): rocky-cloud-%.x86_64.qcow2: $(GARDEN_DL_DIR)/rocky/rocky-cloud-%.x86_64.qcow2 rocky-cloud-%.seed.iso rocky-cloud-%.x86_64.efi-code.img rocky-cloud-%.x86_64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(foreach R,$(ROCKY_RELEASES),rocky-cloud-$R.aarch64.qcow2): rocky-cloud-%.aarch64.qcow2: $(GARDEN_DL_DIR)/rocky/rocky-cloud-%.aarch64.qcow2 rocky-cloud-%.seed.iso rocky-cloud-%.aarch64.efi-code.img rocky-cloud-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

$(GARDEN_DL_DIR)/rocky: | $(GARDEN_DL_DIR)
	mkdir -p $@

rocky-cloud-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,rocky-$*)
rocky-cloud-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

rocky-cloud-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,ROCKY,$*),rocky-$*,rocky)
rocky-cloud-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(ROCKY_RELEASES),$(foreach A,x86_64 aarch64,rocky-cloud-$R.$A.qcow2 rocky-cloud-$R.$A.run))
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
UBUNTU_RELEASES += 14.04

# FIXME: https://gitlab.com/zygoon/image-garden/-/issues/3
# .PHONY: all
# all:: ubuntu-cloud-14.04.$(GARDEN_ARCH).qcow2 ubuntu-cloud-14.04.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/ubuntu/trusty-server-cloudimg-$(call map_to_debian_arch,$(GARDEN_ARCH))-uefi1.img

$(GARDEN_DL_DIR)/ubuntu/trusty-server-cloudimg-amd64-uefi1.img $(GARDEN_DL_DIR)/ubuntu/trusty-server-cloudimg-arm64-uefi1.img: $(GARDEN_DL_DIR)/ubuntu/trusty-server-cloudimg-%-uefi1.img: | $(GARDEN_DL_DIR)/ubuntu
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud-images.ubuntu.com/trusty/current/trusty-server-cloudimg-$*-uefi1.img)

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-14.04.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/trusty-server-cloudimg-amd64-uefi1.img
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-14.04.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/trusty-server-cloudimg-arm64-uefi1.img
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-14.04.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/trusty-server-cloudimg-*-uefi1.img
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
UBUNTU_RELEASES += 16.04

.PHONY: all
all:: ubuntu-cloud-16.04.$(GARDEN_ARCH).qcow2 ubuntu-cloud-16.04.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/ubuntu/xenial-server-cloudimg-$(call map_to_debian_arch,$(GARDEN_ARCH))-uefi1.img

$(GARDEN_DL_DIR)/ubuntu/xenial-server-cloudimg-amd64-uefi1.img $(GARDEN_DL_DIR)/ubuntu/xenial-server-cloudimg-arm64-uefi1.img: $(GARDEN_DL_DIR)/ubuntu/xenial-server-cloudimg-%-uefi1.img: | $(GARDEN_DL_DIR)/ubuntu
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud-images.ubuntu.com/xenial/current/xenial-server-cloudimg-$*-uefi1.img)

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-16.04.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/xenial-server-cloudimg-amd64-uefi1.img
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-16.04.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/xenial-server-cloudimg-arm64-uefi1.img
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-16.04.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/xenial-server-cloudimg-*-uefi1.img
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
UBUNTU_RELEASES += 18.04

.PHONY: all
all:: ubuntu-cloud-18.04.$(GARDEN_ARCH).qcow2 ubuntu-cloud-18.04.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/ubuntu/bionic-server-cloudimg-$(call map_to_debian_arch,$(GARDEN_ARCH)).img

$(GARDEN_DL_DIR)/ubuntu/bionic-server-cloudimg-amd64.img $(GARDEN_DL_DIR)/ubuntu/bionic-server-cloudimg-arm64.img: $(GARDEN_DL_DIR)/ubuntu/bionic-server-cloudimg-%.img: | $(GARDEN_DL_DIR)/ubuntu
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud-images.ubuntu.com/bionic/current/bionic-server-cloudimg-$*.img)

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-18.04.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/bionic-server-cloudimg-amd64.img
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-18.04.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/bionic-server-cloudimg-arm64.img
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-18.04.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/bionic-server-cloudimg-*.img
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
UBUNTU_RELEASES += 20.04

.PHONY: all
all:: ubuntu-cloud-20.04.$(GARDEN_ARCH).qcow2 ubuntu-cloud-20.04.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/ubuntu/focal-server-cloudimg-$(call map_to_debian_arch,$(GARDEN_ARCH)).img

$(GARDEN_DL_DIR)/ubuntu/focal-server-cloudimg-amd64.img $(GARDEN_DL_DIR)/ubuntu/focal-server-cloudimg-arm64.img: $(GARDEN_DL_DIR)/ubuntu/focal-server-cloudimg-%.img: | $(GARDEN_DL_DIR)/ubuntu
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud-images.ubuntu.com/focal/current/focal-server-cloudimg-$*.img)

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-20.04.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/focal-server-cloudimg-amd64.img
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-20.04.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/focal-server-cloudimg-arm64.img
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-20.04.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/focal-server-cloudimg-*.img
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
UBUNTU_RELEASES += 22.04

.PHONY: all
all:: ubuntu-cloud-22.04.$(GARDEN_ARCH).qcow2 ubuntu-cloud-22.04.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/ubuntu/jammy-server-cloudimg-$(call map_to_debian_arch,$(GARDEN_ARCH)).img

$(GARDEN_DL_DIR)/ubuntu/jammy-server-cloudimg-amd64.img $(GARDEN_DL_DIR)/ubuntu/jammy-server-cloudimg-arm64.img: $(GARDEN_DL_DIR)/ubuntu/jammy-server-cloudimg-%.img: | $(GARDEN_DL_DIR)/ubuntu
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud-images.ubuntu.com/jammy/current/jammy-server-cloudimg-$*.img)

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-22.04.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/jammy-server-cloudimg-amd64.img
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-22.04.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/jammy-server-cloudimg-arm64.img
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-22.04.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/jammy-server-cloudimg-*.img
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
UBUNTU_RELEASES += 24.04

.PHONY: all
all:: ubuntu-cloud-24.04.$(GARDEN_ARCH).qcow2 ubuntu-cloud-24.04.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/ubuntu/noble-server-cloudimg-$(call map_to_debian_arch,$(GARDEN_ARCH)).img

$(GARDEN_DL_DIR)/ubuntu/noble-server-cloudimg-amd64.img $(GARDEN_DL_DIR)/ubuntu/noble-server-cloudimg-arm64.img: $(GARDEN_DL_DIR)/ubuntu/noble-server-cloudimg-%.img: | $(GARDEN_DL_DIR)/ubuntu
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud-images.ubuntu.com/noble/current/noble-server-cloudimg-$*.img)

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-24.04.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/noble-server-cloudimg-amd64.img
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-24.04.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/noble-server-cloudimg-arm64.img
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-24.04.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/noble-server-cloudimg-*.img
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
UBUNTU_RELEASES += 24.10

.PHONY: all
all:: ubuntu-cloud-24.10.$(GARDEN_ARCH).qcow2 ubuntu-cloud-24.10.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/ubuntu/oracular-server-cloudimg-$(call map_to_debian_arch,$(GARDEN_ARCH)).img

$(GARDEN_DL_DIR)/ubuntu/oracular-server-cloudimg-amd64.img $(GARDEN_DL_DIR)/ubuntu/oracular-server-cloudimg-arm64.img: $(GARDEN_DL_DIR)/ubuntu/oracular-server-cloudimg-%.img: | $(GARDEN_DL_DIR)/ubuntu
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud-images.ubuntu.com/oracular/current/oracular-server-cloudimg-$*.img)

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-24.10.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/oracular-server-cloudimg-amd64.img
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-24.10.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/oracular-server-cloudimg-arm64.img
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-24.10.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/oracular-server-cloudimg-*.img
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
UBUNTU_RELEASES += 25.04

.PHONY: all
all:: ubuntu-cloud-25.04.$(GARDEN_ARCH).qcow2 ubuntu-cloud-25.04.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/ubuntu/plucky-server-cloudimg-$(call map_to_debian_arch,$(GARDEN_ARCH)).img

$(GARDEN_DL_DIR)/ubuntu/plucky-server-cloudimg-amd64.img $(GARDEN_DL_DIR)/ubuntu/plucky-server-cloudimg-arm64.img: $(GARDEN_DL_DIR)/ubuntu/plucky-server-cloudimg-%.img: | $(GARDEN_DL_DIR)/ubuntu
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud-images.ubuntu.com/plucky/current/plucky-server-cloudimg-$*.img)

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-25.04.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/plucky-server-cloudimg-amd64.img
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-25.04.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/plucky-server-cloudimg-arm64.img
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-25.04.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/plucky-server-cloudimg-*.img
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
UBUNTU_RELEASES += 25.10

.PHONY: all
all:: ubuntu-cloud-25.10.$(GARDEN_ARCH).qcow2 ubuntu-cloud-25.10.$(GARDEN_ARCH).run

.PHONY: fetch
fetch: $(GARDEN_DL_DIR)/ubuntu/questing-server-cloudimg-$(call map_to_debian_arch,$(GARDEN_ARCH)).img

$(GARDEN_DL_DIR)/ubuntu/questing-server-cloudimg-amd64.img $(GARDEN_DL_DIR)/ubuntu/questing-server-cloudimg-arm64.img: $(GARDEN_DL_DIR)/ubuntu/questing-server-cloudimg-%.img: | $(GARDEN_DL_DIR)/ubuntu
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cloud-images.ubuntu.com/questing/current/questing-server-cloudimg-$*.img)

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-25.10.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/questing-server-cloudimg-amd64.img
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-25.10.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu $(GARDEN_DL_DIR)/ubuntu/questing-server-cloudimg-arm64.img
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-25.10.*.qcow2

.PHONY: distclean
distclean::
	rm -f $(GARDEN_DL_DIR)/ubuntu/questing-server-cloudimg-*.img
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki

UBUNTU_CORE_RELEASES = 16 18 20 22 24
GARDEN_SYSTEMS += $(foreach r,$(UBUNTU_CORE_RELEASES),$(foreach a,x86_64 aarch64,ubuntu-core-$r.$a))

$(foreach R,$(UBUNTU_CORE_RELEASES),ubuntu-core-$R.x86_64.run): ubuntu-core-%.x86_64.run: $(MAKEFILE_LIST) | ubuntu-core-%.x86_64.qcow2 ubuntu-core-%.x86_64.efi-code.img ubuntu-core-%.x86_64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/ubuntu-core" >>$@
	echo '$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(UBUNTU_CORE_RELEASES),ubuntu-core-$R.aarch64.run): ubuntu-core-%.aarch64.run: $(MAKEFILE_LIST) | ubuntu-core-%.aarch64.qcow2 ubuntu-core-%.aarch64.efi-code.img ubuntu-core-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/ubuntu-core" >>$@
	echo '$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(UBUNTU_CORE_RELEASES),ubuntu-core-$R.x86_64.qcow2): ubuntu-core-%.x86_64.qcow2: $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-%.x86_64.qcow2 ubuntu-core-%.seed.iso ubuntu-core-%.x86_64.efi-code.img ubuntu-core-%.x86_64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-scsi-pci,id=scsi1 \
		-device scsi-cd,drive=drive1,bus=scsi1.0 \
		| tee $@.log)

$(foreach R,$(UBUNTU_CORE_RELEASES),ubuntu-core-$R.aarch64.qcow2): ubuntu-core-%.aarch64.qcow2: $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-%.aarch64.qcow2 ubuntu-core-%.seed.iso ubuntu-core-%.aarch64.efi-code.img ubuntu-core-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-scsi-pci,id=scsi1 \
		-device scsi-cd,drive=drive1,bus=scsi1.0 \
		| tee $@.log)

$(GARDEN_DL_DIR)/ubuntu-core: | $(GARDEN_DL_DIR)
	mkdir -p $@

ubuntu-core-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,ubuntu-core-$*)
ubuntu-core-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

ubuntu-core-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,UBUNTU_CORE,$*),ubuntu-core-$*,ubuntu)
ubuntu-core-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

define BASE_UBUNTU_CORE_CLOUD_INIT_USER_DATA_TEMPLATE
#cloud-config
datasource_list: [NoCloud]
timezone: Europe/Warsaw
power_state:
  delay: now
  mode: poweroff
  message: Powering off
  timeout: 0
  condition: true
hostname: $(subst .,-,$1)
manage_etc_hosts: true
ssh_pwauth: true
runcmd:
# Postpone refreshes for a day.
- snap set system refresh.hold="$$(date --date=tomorrow +%Y-%m-%dT%H:%M:%S%:z)"
# Bring the network down so that ongoing refresh can fail.
- ip link set dev eth0 down || true
# Create the user account.
- useradd --comment "$2 User" --create-home --shell /bin/bash --extrausers $2
- echo "$2:$2" | chpasswd $2
- echo image-garden >/writable/system-data/var/lib/console-conf/complete
# Wait for snapd to finish seeding.
- snap wait system seed.loaded
# Abort the change to refresh core and kernel snaps that may have started.
- snap abort 3 || true
# Applicable to Ubuntu Core 24+ where console-conf is a snap.
- snap remove --purge console-conf || true
# Restore network connectivity.
- ip link set dev eth0 up || true
endef

UBUNTU_CORE_CLOUD_INIT_USER_DATA_TEMPLATE=$(BASE_UBUNTU_CORE_CLOUD_INIT_USER_DATA_TEMPLATE)

ubuntu-core-16.x86_64.qcow2 ubuntu-core-16.x86_64.run ubuntu-core-18.x86_64.qcow2 ubuntu-core-18.x86_64.run: QEMU_ENV_QUIRKS=export QEMU_STORAGE_OPTION="$(strip \
	-drive file=$(1),if=none,format=qcow2,id=drive0,media=disk,cache=writeback,discard=unmap \
	-device virtio-scsi-pci,id=scsi0 \
	-device scsi-hd,drive=drive0,bus=scsi0.0,bootindex=0)";

.PHONY: all
all:: $(foreach R,$(filter-out 16 18 20,$(UBUNTU_CORE_RELEASES)),ubuntu-core-$R.$(GARDEN_ARCH).qcow2 ubuntu-core-$R.$(GARDEN_ARCH).run)
ifeq ($(GARDEN_ARCH),x86_64)
all:: $(foreach R,$(filter 16 18 20,$(UBUNTU_CORE_RELEASES)),ubuntu-core-$R.$(GARDEN_ARCH).qcow2 ubuntu-core-$R.$(GARDEN_ARCH).run)
fetch: $(foreach R,$(filter 16 18 20,$(UBUNTU_CORE_RELEASES)),$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-$R-$(call map_to_debian_arch,$(GARDEN_ARCH)).img.xz)
endif

.PHONY: fetch
fetch: $(foreach R,$(filter-out 16 18,$(UBUNTU_CORE_RELEASES)),$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-$R-$(call map_to_debian_arch,$(GARDEN_ARCH)).img.xz)

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-16-amd64.img.xz: | $(GARDEN_DL_DIR)/ubuntu-core
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cdimage.ubuntu.com/ubuntu-core/16/stable/current/ubuntu-core-16-amd64.img.xz)

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-18-amd64.img.xz: | $(GARDEN_DL_DIR)/ubuntu-core
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cdimage.ubuntu.com/ubuntu-core/18/stable/current/ubuntu-core-18-amd64.img.xz)

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-20-amd64.img.xz $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-20-arm64.img.xz: $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-20-%.img.xz: | $(GARDEN_DL_DIR)/ubuntu-core
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cdimage.ubuntu.com/ubuntu-core/20/stable/current/ubuntu-core-20-$*.img.xz)

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-22-amd64.img.xz $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-22-arm64.img.xz: $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-22-%.img.xz: | $(GARDEN_DL_DIR)/ubuntu-core
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cdimage.ubuntu.com/ubuntu-core/22/stable/current/ubuntu-core-22-$*.img.xz)

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-24-amd64.img.xz $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-24-arm64.img.xz: $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-24-%.img.xz: | $(GARDEN_DL_DIR)/ubuntu-core
	$(strip $(WGET) \
		--no-clobber \
		--directory-prefix=$(firstword $|)/ \
		https://cdimage.ubuntu.com/ubuntu-core/24/stable/current/ubuntu-core-24-$*.img.xz)

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-16.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu-core $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-16-amd64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-18.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu-core $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-18-amd64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-20.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu-core $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-20-amd64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-22.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu-core $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-22-amd64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-22.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu-core $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-22-arm64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-24.x86_64.qcow2: | $(GARDEN_DL_DIR)/ubuntu-core $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-24-amd64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-24.aarch64.qcow2: | $(GARDEN_DL_DIR)/ubuntu-core $(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-24-arm64.qcow2
	ln -sf $(notdir $(word 2,$|)) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(UBUNTU_CORE_RELEASES),$(foreach A,x86_64 aarch64,ubuntu-core-$R.$A.qcow2 ubuntu-core-$R.$A.run))

.PHONY: clean
clean::
	rm -f $(foreach R,$(UBUNTU_CORE_RELEASES),$(GARDEN_DL_DIR)/ubuntu-core/ubuntu-core-$R.*.qcow2)

.PHONY: distclean
distclean::
	rm -f $(foreach R,$(UBUNTU_CORE_RELEASES),$(GARDEN_DL_DIR)/ubuntu/ubuntu-core-$R-*.img.xz)
# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: Zygmunt Krynicki
UBUNTU_RELEASES ?=
GARDEN_SYSTEMS += $(foreach r,$(UBUNTU_RELEASES),$(foreach a,x86_64 aarch64,ubuntu-cloud-$r.$a))

$(foreach R,$(UBUNTU_RELEASES),ubuntu-cloud-$R.x86_64.run): ubuntu-cloud-%.x86_64.run: $(MAKEFILE_LIST) | ubuntu-cloud-%.x86_64.qcow2 ubuntu-cloud-%.x86_64.efi-code.img ubuntu-cloud-%.x86_64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/ubuntu" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

$(foreach R,$(UBUNTU_RELEASES),ubuntu-cloud-$R.aarch64.run): ubuntu-cloud-%.aarch64.run: $(MAKEFILE_LIST) | ubuntu-cloud-%.aarch64.qcow2 ubuntu-cloud-%.aarch64.efi-code.img ubuntu-cloud-%.aarch64.efi-vars.img
	echo "#!/bin/sh" >$@
	echo 'set -xeu' >>$@
	echo "# WARNING: The .qcow2 file refers to a file in $(GARDEN_DL_DIR)/ubuntu" >>$@
	echo '$(strip exec $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$(word 1,$|),$(word 2,$|),$(word 3,$|)) "$$@")' >>$@
	chmod +x $@

# Ubuntu cloud images do not contain a standalone installer that you only need
# during the setup process. Instead each image can be used as a read-only base
# for a read-write template.
$(foreach R,$(UBUNTU_RELEASES),ubuntu-cloud-$R.x86_64.qcow2): ubuntu-cloud-%.x86_64.qcow2: $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-%.x86_64.qcow2 ubuntu-cloud-%.seed.iso ubuntu-cloud-%.x86_64.efi-code.img ubuntu-cloud-%.x86_64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_X86_64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

# Replicated because we have more releases than architectures and architecture-specific quirks are easier to express.
$(foreach R,$(UBUNTU_RELEASES),ubuntu-cloud-$R.aarch64.qcow2): ubuntu-cloud-%.aarch64.qcow2: $(GARDEN_DL_DIR)/ubuntu/ubuntu-cloud-%.aarch64.qcow2 ubuntu-cloud-%.seed.iso ubuntu-cloud-%.aarch64.efi-code.img ubuntu-cloud-%.aarch64.efi-vars.img
	$(strip $(QEMU_IMG) create -f qcow2 -b $< -F qcow2 $@ $(QEMU_IMG_SIZE))
	$(strip $(call QEMU_SYSTEM_AARCH64_EFI_CMDLINE,$@,$(word 3,$^),$(word 4,$^)) \
		-drive file=$(word 2,$^),format=raw,id=drive1,if=none,readonly=true,media=cdrom \
		-device virtio-blk,drive=drive1 \
		-no-reboot | tee $@.log)

# Ubuntu specific downloads.
$(GARDEN_DL_DIR)/ubuntu: | $(GARDEN_DL_DIR)
	mkdir -p $@

ubuntu-cloud-%.meta-data: export META_DATA = $(call CLOUD_INIT_META_DATA_TEMPLATE,ubuntu-$*)
ubuntu-cloud-%.meta-data: $(MAKEFILE_LIST)
	echo "$${META_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

ubuntu-cloud-%.user-data: export USER_DATA = $(call $(call PICK_CLOUD_INIT_USER_DATA_TEMPLATE_FOR,UBUNTU,$*),ubuntu-$*,ubuntu)
ubuntu-cloud-%.user-data: $(MAKEFILE_LIST) $(wildcard .image-garden.mk)
	echo "$${USER_DATA}" | tee $@
	touch --reference=$(shell stat $^ -c '%Y %n' | sort -nr | cut -d ' ' -f 2 | head -n 1) $@

.PHONY: clean
clean::
	rm -f $(foreach R,$(UBUNTU_RELEASES),$(foreach A,x86_64 aarch64,ubuntu-cloud-$R.$A.qcow2 ubuntu-cloud-$R.$A.run))
